@@ -2,14 +2,17 @@ package fzf
 
 import (
 	"bufio"
+	"context"
 	"io"
 	"os"
 	"os/exec"
+	"path/filepath"
 	"sync"
 	"sync/atomic"
 	"time"
 
 	"github.com/junegunn/fzf/src/util"
+	"github.com/saracen/walker"
 )
 
 // Reader reads from command or standard input
@@ -78,7 +81,7 @@ func (r *Reader) terminate() {
 	r.killed = true
 	if r.exec != nil && r.exec.Process != nil {
 		util.KillCommand(r.exec)
-	} else {
+	} else if defaultCommand != "" {
 		os.Stdin.Close()
 	}
 }
@@ -99,7 +102,11 @@ func (r *Reader) ReadSource() {
 		shell := "bash"
 		cmd := os.Getenv("FZF_DEFAULT_COMMAND")
 		if len(cmd) == 0 {
-			success = r.readFromCommand(&shell, defaultCommand)
+			if defaultCommand != "" {
+				success = r.readFromCommand(&shell, defaultCommand)
+			} else {
+				success = r.readFiles()
+			}
 		} else {
 			success = r.readFromCommand(nil, cmd)
 		}
@@ -144,6 +151,32 @@ func (r *Reader) readFromStdin() bool {
 	return true
 }
 
+func (r *Reader) readFiles() bool {
+	r.killed = false
+	fn := func(path string, mode os.FileInfo) error {
+		path = filepath.Clean(path)
+		if path != "." {
+			isDir := mode.Mode().IsDir()
+			if isDir && filepath.Base(path)[0] == '.' {
+				return filepath.SkipDir
+			}
+			if !isDir && r.pusher([]byte(path)) {
+				atomic.StoreInt32(&r.event, int32(EvtReadNew))
+			}
+		}
+		r.mutex.Lock()
+		defer r.mutex.Unlock()
+		if r.killed {
+			return context.Canceled
+		}
+		return nil
+	}
+	cb := walker.WithErrorCallback(func(pathname string, err error) error {
+		return nil
+	})
+	return walker.Walk(".", fn, cb) == nil
+}
+
 func (r *Reader) readFromCommand(shell *string, command string) bool {
 	r.mutex.Lock()
 	r.killed = false
